import React, { useEffect, useState } from "react";
import { Table ,Switch} from 'antd';
import EditUnit from "./comm";
import useMount from "../useMount"
import { isObject, guid } from "../util";
import { SearchOutlined } from '@ant-design/icons';
import "./index.less"
// 数据仓库 
const editTableState = {};
const action = {};
let initData = [
    {
        key: '1',
        name: 'John Brown',
        age: 32,
        address: 'New York No. 1 Lake Park',
        tags: ['nice', 'developer'],
        "range_picker": "2014-11-21至2015-10-09"
    }
]
let columns = [
    {
        title: '测试输入框',
        dataIndex: 'name',
        key: 'name',
        isEdit: true,
        type: "input",
        itemConfig: {
            rules: [{
                required: true,
                message: `测试输入框 不能为空`,

            },
            {
                pattern: new RegExp(/^\d+$|^\d*\.\d+$/, "g"),
                message: '只能输入数字'
            }]
        }
        // render: ()=>{
        //     return <Input />
        // }, TextArea
    },
    {
        title: '测试下拉选择',
        type: "select",
        options: [
            {
                value: "123",
                label: "1111"
            }
        ],
        isEdit: true,
        dataIndex: 'age',
        key: 'age',
    },
    {
        title: '测试文本',
        type: "textArea",
        isEdit: true,
        dataIndex: 'text',
        key: 'text',
    },
    {
        title: '测试时间范围选择',
        type: "rangePicker",
        isEdit: true,
        dataIndex: 'range_picker',
        key: 'range_picker',
    }
]
// 获取表格数据  整表 或者整行 或者相应的key
const menGetEditTableData = (id, row, key) => {
    if (!action[id]) {
        console.warn ("EditTable menGetEditTableData parameter  illegality !Mendege")
    }
    let data = null;
    data = action[id].getData(id);
    if (!row) {
        return data||[];
    }
    data = data[row] || null;
    if (!key) {
        return data;
    }

    return data[row][key] || [];
}
// 设置表格数据  设置全量数据
const menSetEditTableData = (id, data) => {
    if (!Array.isArray(data)) {
        console.warn (" menSetEditTableData 数据格式错误 !Mendege")
    }
    if (action[id] && action[id].setData && action[id].setData) {
        let newData = [];
        data.forEach((obj, k) => {
            let i = { ...obj };
            i.key = k;
            newData.push(i)
        })
        // 需要校验数据合法性
        action[id] && action[id].setData && action[id].setData(newData || []);
    } else {
        console.warn ("menSetEditTableData 当前设置时机有问题 实例不存在 或者 传入id错误 !Mendege")
    }
}
// 设置表格数据 行 & 列 & key if index === all 并且 key 存在的情况下 设置整列数据 
const menSetEditTableRowData = (id, con) => {
    try {
        if (action[id] && action[id].setData && action[id].getData) {
            let { index, key, data } = con;
            let sdata = action[id].getData();
            if (typeof index === "number") {
                // key 不存在 设置整行数据 
                if (!key) {
                    if (isObject(data)) {
                        sdata.some((i, k) => {
                            if (k === index) {
                                i = { ...i, ...data, key: i.key }
                                return true;
                            }
                        });
                        action[id] && action[id].setData && action[id].setData(sdata || []);
                        return;
                    } else {
                        console.warn ("设置表格行数据 需要是对象 !Mendege")
                    }
                } else {
                    //key 存在  设置某行某列的数据
                    sdata.some((i, k) => {
                        if (k === index) {
                            i[key] = data;
                            return true;
                        }
                    });
                    action[id] && action[id].setData && action[id].setData(sdata || []);
                    return;
                }

            }
            if (index === "all" && key) {
                let newData = [];
                sdata.forEach((i, k) => {
                    newData.push({ ...i, [key]: data })
                });
                action[id] && action[id].setData && action[id].setData(newData || []);
                return
            }
            console.warn ("menSetEditTableRowData 数据设置未成功 !Mendege")
        } else {
            console.warn ("menSetEditTableRowData 当前设置时机有问题 实例不存在 或者 传入id错误 !Mendege")
        }
    } catch (error) {
        console.warn ("menSetEditTableRowData 当前设置时机有问题 实例不存在 或者 传入id错误 !Mendege" + error);
    }

}
// 设置表格模板
const menSetEditTableMeta = (id, data) => {
    // 需要校验数据合法性
    
    if(action[id] && action[id].setData && action[id].setMeta){
        action[id] && action[id].setMeta && action[id].setMeta(data);
    }else{
        console.warn ("menSetEditTableMeta 当前设置时机不对 实例不存在 或者 传入id错误 !Mendege")
    }
}
// 获取表格模板
const menGetEditTableMeta = (id, data) => {
    if (!action[id]) {
        console.warn ("EditTable menGetEditTableData parameter  illegality !Mendege")
    }
    return action[id].getMeta(id) || null;
}
// 增行  data object 单行  number n个空行 array 对应对行  对象或者数组 必须要有相应数据项  和模板能一一对应
// flag true 往后添加  默认  false 往前添加
const menAddEditTableRow = (id, data, flag = true) => {
    let newRow = [];
    let sdata = menGetEditTableData(id);
    let meta = menGetEditTableMeta(id);
    if (typeof data === "number") {
        // newRow = [data[0]]
        if (meta && meta.items && Array.isArray(meta.items)) {
            try {
                let obj = {};
                meta.items.forEach((i) => {
                    obj[i.dataIndex] = null;
                });
                for (let i = 0; i < data; i++) {
                    if (flag) {
                        sdata.push(obj);
                    } else {
                        sdata.unshift(obj);
                    }
                }
                menSetEditTableData(id, sdata);
            } catch (error) {
                console.warn ("menAddEditTableRow 发生错误：" + error)
            }

        } else {
            console.warn ("EditTable menAddEditTableRow meta parameter  illegality !Mendege")
        }
    } else if (typeof data === "undefined") {

        try {
            let obj = {};
            meta.items.forEach((i) => {
                obj[i.dataIndex] = null;
            });
            if (flag) {
                sdata.push(obj);
            } else {
                sdata.unshift(obj);
            }
            menSetEditTableData(id, sdata);
        } catch (error) {
            console.warn ("menAddEditTableRow 发生错误：" + error)
        }

    } else if (Array.isArray(data)) {
        try {
            let obj = {};
            data.items.forEach((i) => {
                if (flag) {
                    sdata.push(i);
                } else {
                    sdata.unshift(i);
                }
            });
            menSetEditTableData(id, sdata);
        } catch (error) {
            console.warn ("menAddEditTableRow 发生错误：" + error)
        }
    } else if (isObject(data)) {
        try {
            if (flag) {
                sdata.push(data);
            } else {
                sdata.unshift(data);
            }
            menSetEditTableData(id, sdata);
        } catch (error) {
            console.warn ("menAddEditTableRow 发生错误：" + error)
        }
    } else {
        console.warn ("EditTable menAddEditTableRow parameter  illegality !Mendege")
    }
}
// 删除行 
const menDeleteEditTableRowIndex = (id,index)=>{
    let data = menGetEditTableData(id);
    data = data.filter(i=>i.key!=index);
    menSetEditTableData(id,data);
}

// 表格校验
const menVerifyEditTable = (id)=>{
    return new Promise((reslove,reject)=>{
        Promise.all([...action[id].refs.values()].map(i=>i.validateFields())).then((res)=>{
            reslove(true)
        },(error)=>{
            reject(error)
        })
    });
   
}
// 获取当前页面所有的 可编辑表格 的action
const menGetAllEditTable = ()=>{
    return action;
}
// 收集action
const setInstance = (id,key,instancetem,del=false)=>{
    action[id]&&action[id].refs.set(key,instancetem)
}
// 开关型数据列

// 创建表格区域  没有meta 无法创建
/* 
    disposeMeta:fn,//处理模板 最后一步 
*/
const MenCreateEditTableSon = ({id, config = {}}) => {
    if(!action[id]){
        action[id] = {
            checkedMap : new Map()
        };
    }
    
    let { tableConfig = {} } = config;
    let [dataSource, setDataSource] = useState(config.data||[]);
    let [meta, setMeta] = useState(config.meta||{});
    action[id].setData = (res) => {
        setDataSource([...res])
    };
    action[id].setMeta = (res) => {
        setMeta({ ...res });
    };
    action[id].getMeta = () => {
        return meta;
    };
    action[id].getData = () => {
        return dataSource;
    };
    if(!action[id].refs){
        action[id].refs = new Map();
    }
    
    // 首次渲染
    useMount(()=>{
        return ()=>{
            delete action[id]
            // alert("组件被卸载")
        }
    })
    // 父组件如果更新 那么需要更新子组件的值 
    // useEffect(() => {
    //     setDataSource(props.data)
    // }, [props.data])
    // 编辑后事件 key,当前编辑的 key/dataIndex  lastVal 上一次的值 current 当前的值  record 当前行信息 暂时 不开发
    const afterChange = (key, lastVal, current, record) => {

    }
    // 编辑前事件 会检查父组件 是否传递该方法  如果传递了该方法 
    // 那么会获取其返回值 必须返回有true 组件才会继续编辑 否则 组件无法编辑 会一直维持旧值
    // key 当前编辑的 key/dataIndex current 当前单元格值  record 当前行信息 action 表单实例
    const beforeChange = (key, current, record, action) => {
        let falg = true;
        if (config.beforeChange) {
            falg = config.beforeChange(current, record, action);
        }
        return falg;
    }
    // setChange   值改变事件 key,当前编辑的 key/dataIndex  index 行索引 lastVal 上一次的值 current 当前的值  record 当前行信息 action 表单实例 
    const setChange = (key, index, current, record, action) => {
        menSetEditTableRowData(id, {
            index,
            key,
            data: current
        });
        // 每次数据更新需要验证 是否属于 checked 如果是  需要 联动列头上的复选框

        config.setChange && config.setChange(id, key, index, current, record)
    }

    let newColumns = [];
    // meta.items = newColumns;
    // return <div>不试一下么</div>
   
    if(!meta.items){
        return null;
    }
    if(config.disposeMeta){
        newColumns = config.disposeMeta([...meta.items])
    }else{
        newColumns = [...meta.items];
    }
    let metaItems = []
    newColumns && newColumns.forEach((i, k) => {
        let render = {};
        let addClassName = " ";
        if (i.render) {
            render = { render: i.render };
        } else if (i.isEdit && meta.status) {
            
            render = {
                render: (text, record, index) => {
                    return <EditUnit
                        setInstance = {(key,instanceItem,del=false)=>{
                            setInstance(id,key,instanceItem,del)
                        }}
                        key={k + i.type}
                        val={text}
                        index={index}
                        type={i.type}
                        dataIndex={i.dataIndex}
                        setChange={setChange}
                        beforeChange={beforeChange}
                        itemConfig={i.itemConfig}
                        record={record}
                        options={i.options}
                        is_number={i.is_number}

                    ></EditUnit>
                }
            };
            if(i.type==="switch"){
                let checkAll = true;
                // 如果是开关的情况下  每次渲染都需要 统计当前列数据是否为全量数据
                dataSource.forEach((y)=>{
                    if(!y[i.dataIndex]){
                        checkAll = false;
                    }
                })
                addClassName+=" switch_cell"
                render.filterDropdown= ({ setSelectedKeys, selectedKeys, confirm, clearFilters }) => (
                    <div>
                    </div>
                  );
                  render.filterIcon= filtered => <Switch onChange={(e)=>{
                    menSetEditTableRowData(id, {
                        index:"all",
                        key:i.dataIndex,
                        data: e
                    });
                    // 每次数据更新需要验证 是否属于 checked 如果是  需要 联动列头上的复选框
            
                    config.setChange && config.setChange(id, i.dataIndex, "all", e,{})
                  }} checked={
                    checkAll
                    } />;
                  render.align = "right";
            }
        } else {
            render = {
                render: (text, record, index) => {
                    let className = "title";
                    if(i.wr_ellipsis){
                        className  +=" wr_ellipsis"
                    }
                    if (record[i.dataIndex + "Name"]) {
                        return record[i.dataIndex + "Name"]
                    } else {
                        return <span style={{width:i?.width}} title={text} className={className}>{text}</span>;
                    }

                }
            }
        }
        let width = {};
        if (i.width) {
            width = {
                width: i.width
            }
        }
        metaItems.push({
            ...width,
            title: i.title,
            dataIndex: i.dataIndex,
            key: i.key||k,
            onCell: i.onCell,

            className: i.className+addClassName,
            ...render

        });
    })
    if(config["serial-number"]){
        metaItems.unshift({
            title: '序号',
            dataIndex: 'row-index',
            type: "input",
            width:80,
            render:(t,r,index)=>{
                return index+1
            }
      })
    }
    return <Table columns={metaItems || []} dataSource={dataSource || []} {...tableConfig} />
}

const menCreateEditTable = (id,config)=>{
    return <div className="men-edit-tabl">
                <MenCreateEditTableSon {...{id,config}}></MenCreateEditTableSon>
    </div>
}




export default {
    menSetEditTableData, menCreateEditTable,
    menSetEditTableMeta, menAddEditTableRow,
    menGetEditTableData, menSetEditTableRowData,
    menGetEditTableMeta,menDeleteEditTableRowIndex,
    menVerifyEditTable,menGetAllEditTable
}